/**
 * @file cuda_stubs.h
 * @brief Stubs for a no-CUDA build.
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.utk.edu/pulsar/
 * Copyright (C) 2012-2015 University of Tennessee.
 */

#ifndef CUDA_STUBS_H
#define CUDA_STUBS_H

#ifndef CUDA

#include <stdio.h>
#include <stdlib.h>

#define CUDART_CB
#define __cudart_builtin__

typedef int cudaError_t;
typedef int cudaEvent_t;
typedef int cudaStream_t;
typedef int cudaMemcpyKind;
typedef void CUDART_CB (*cudaStreamCallback_t)(cudaStream_t, cudaError_t, void*);
enum {cudaSuccess, cudaMemcpyHostToDevice, cudaMemcpyDeviceToHost, cudaStreamNonBlocking};

#ifdef __cplusplus
extern "C" {
#endif

cudaError_t cudaSetDevice(int device);
cudaError_t cudaGetDevice(int* device);
cudaError_t cudaFree(void* devPtr);
cudaError_t cudaMalloc(void** devPtr, size_t size);
cudaError_t cudaMemGetInfo(size_t* free, size_t* total);
cudaError_t cudaStreamDestroy(cudaStream_t stream);     
cudaError_t cudaStreamCreateWithFlags(cudaStream_t* pStream, unsigned int flags);
cudaError_t cudaStreamAddCallback(cudaStream_t stream, cudaStreamCallback_t callback, void* userData, unsigned int flags);
cudaError_t cudaEventCreate(cudaEvent_t* event);
cudaError_t cudaEventRecord(cudaEvent_t event, cudaStream_t stream);
cudaError_t cudaEventElapsedTime(float* ms, cudaEvent_t start, cudaEvent_t end);
cudaError_t cudaMemcpyAsync(void* dst, const void* src, size_t count, cudaMemcpyKind kind, cudaStream_t stream);
cudaError_t cudaMemcpyPeerAsync(void* dst, int dstDevice, const void* src, int srcDevice, size_t count, cudaStream_t stream);
cudaError_t cudaDeviceSynchronize(void);
const __cudart_builtin__ char* cudaGetErrorString(cudaError_t error);

#ifdef __cplusplus
}
#endif

#endif /* CUDA */

#endif /* CUDA_STUBS_H */
