/**
 * @file gpu_malloc.h
 * @brief Simple device memory allocator.
 * @author Aurelien Bouteiller
 * @author Thomas Herault
 * @author George Bosilca
 *
 * PULSAR Runtime
 * http://icl.utk.edu/pulsar/
 * Copyright (C) 2012-2015 University of Tennessee.
 */

#ifndef GPU_MALLOC_H
#define GPU_MALLOC_H

#include <stdlib.h>

#ifdef CUDA
    #include <cuda.h>
    #include <cuda_runtime.h>
#else
    #include "cuda_stubs.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef struct segment {
    int start_index;    // Index of the first byte of this segment.
    int nb_units;       // Number of units occupied by this segment.
    int nb_free;        // Number of units free after this segment.
    struct segment *next;
} segment_t;

typedef struct gpu_malloc_s {
    char      *base;                // Base pointer.
    segment_t *allocated_segments;  // List of allocated segments.
    segment_t *free_segments;       // List of available segments.
    size_t     unit_size;           // Unit size.
    int        max_segment;         // Maximum number of segments.
} gpu_malloc_t;

gpu_malloc_t* gpu_malloc_init(int max_segment, size_t unit_size);
int gpu_malloc_fini(gpu_malloc_t *gdata);
void* gpu_malloc(gpu_malloc_t *gdata, size_t size);
int gpu_free(gpu_malloc_t *gdata, void *ptr);

#ifdef __cplusplus
}
#endif

#endif /* GPU_MALLOC_H */
