/**
 * @file mpi_stubs.h
 * @brief Stubs for a no-MPI build.
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.utk.edu/pulsar/
 * Copyright (C) 2012-2015 University of Tennessee.
 */

#ifndef MPI_STUBS_H
#define MPI_STUBS_H

#ifndef MPI

#include <stdio.h>
#include <stdlib.h>

#define MPI_STATUS_IGNORE NULL

typedef int MPI_Op;
typedef int MPI_Comm;
typedef int MPI_Datatype;

typedef struct {
    int MPI_TAG;
    int MPI_SOURCE;
} MPI_Status;

typedef struct {
} MPI_Request;

enum {MPI_BYTE, MPI_INT, MPI_DOUBLE, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_MAX, MPI_SUCCESS, MPI_COMM_WORLD};

#ifdef __cplusplus
extern "C" {
#endif

int MPI_Initialized(int *flag);
int MPI_Barrier(MPI_Comm comm);
int MPI_Cancel(MPI_Request *request);
int MPI_Abort(MPI_Comm comm, int errorcode);
int MPI_Comm_rank(MPI_Comm comm, int *rank);
int MPI_Comm_size(MPI_Comm comm, int *size);
int MPI_Test(MPI_Request *request, int *flag, MPI_Status *status);
int MPI_Get_count(const MPI_Status *status, MPI_Datatype datatype, int *count);
int MPI_Send(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
int MPI_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status);
int MPI_Irecv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);
int MPI_Isend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
int MPI_Reduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm);

#ifdef __cplusplus
}
#endif

#endif /* MPI */

#endif /* MPI_STUBS_H */
