/**
 * @file prt.h
 * @brief PULSAR Runtime (PRT)
 * @author Jakub Kurzak
 *
 * @defgroup API_core PRT API - core interface
 * @defgroup API_aux  PRT API - auxiliary interface
 * @defgroup API_acc  PRT API - accelerator interface
 *
 * PULSAR Runtime
 * http://icl.utk.edu/pulsar/
 * Copyright (C) 2012-2015 University of Tennessee.
 */

#ifndef PRT_H
#define PRT_H

#define _GNU_SOURCE

/**
 * @brief Locations of VDPs and packets.
 */
typedef enum prt_location_e {
    PRT_LOCATION_HOST,
    PRT_LOCATION_DEVICE
} prt_location_t;

/**
 * @brief Directions of local transfers.
 */
typedef enum prt_direction_e {
    PRT_HOST_TO_DEVICE,
    PRT_DEVICE_TO_HOST,
    PRT_DEVICE_TO_DEVICE,
    PRT_DEVICE_MPI_TO_HOST,
    PRT_DEVICE_MPI_FROM_HOST,
    PRT_DEVICE_PACKET_RELEASE
} prt_direction_t;

/**
 * @brief Mapping of VDPs to hardware.
 */
typedef struct prt_mapping_s {
    prt_location_t location;
    int rank;
} prt_mapping_t;

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <stdarg.h>
#include <limits.h>
#include <malloc.h>
#include <sched.h>
#include <pthread.h>

#ifdef MPI
    #include <mpi.h>
#else
    #include "mpi_stubs.h"
#endif

#ifdef CUDA
    #include <cuda.h>
    #include <cuda_runtime.h>
#else
    #include "cuda_stubs.h"
#endif

#include "gpu_malloc.h"
#include "icl_list.h"
#include "icl_hash.h"
#include "icl_deque.h"
#include "svg_trace.h"
#include "prt_assert.h"
#include "prt_tuple.h"
#include "prt_packet.h"
#include "prt_channel.h"
#include "prt_vdp.h"
#include "prt_thread.h"
#include "prt_device.h"
#include "prt_request.h"
#include "prt_transfer.h"
#include "prt_callback.h"
#include "prt_proxy.h"
#include "prt_config.h"
#include "prt_vsa.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

#endif /* PRT_H */
