/**
 * @file prt_assert.c
 * @brief PRT exception handling.
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.utk.edu/pulsar/
 * Copyright (C) 2012-2015 University of Tennessee.
 */

#include "prt_assert.h"

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief Checks an assertion and exits on error.
 * Prints an error message.
 *
 * @param cond -- The condition.
 * @param msg -- The error message.
 * @param line -- The line number.
 * @param file -- The name of the source file.
 */
void prt_assert_line_file(int cond, const char *msg, int line, char *file)
{
    if (!cond)
        prt_error_line_file(msg, line, file);
}

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief Prints an error message and exits.
 *
 * @param msg -- The error message.
 * @param line -- The line number.
 * @param file -- The name of the source file.
 */
void prt_error_line_file(const char *msg, int line, char *file)
{
    fprintf(stderr, "\nPRT ERROR: %s (in %s at line %d)\n\n", msg, file, line);
    fflush(stderr);
    MPI_Abort(MPI_COMM_WORLD, 0);
}

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief Prints a warning and continues.
 *
 * @param msg -- The warning message.
 * @param line -- The line number.
 * @param file -- The name of the source file.
 */
void prt_warning_line_file(const char *msg, int line, char *file)
{
    fprintf(stderr, "\nPRT WARNING: %s (in %s at line %d)\n\n", msg, file, line);
    fflush(stderr);
}
