/**
 * @file prt_callback.h
 * @brief PRT callback.
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.utk.edu/pulsar/
 * Copyright (C) 2012-2015 University of Tennessee.
 */

#ifndef PRT_CALLBACK_H
#define PRT_CALLBACK_H

#include "prt.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Callback data for finishing a local communication.
 */
typedef struct prt_callback_finish_s {
    struct prt_packet_s *src_packet;    // Source packet.
    struct prt_packet_s *dst_packet;    // Destination packet.
    struct prt_channel_s *channel;      // Destination channel.
} prt_callback_finish_t;

/**
 * @brief Callback data for queueing a local communication.
 */
typedef struct prt_callback_queue_s {
    struct prt_packet_s *old_packet;    // Packet to be released.
    struct prt_packet_s *src_packet;    // Packet to send.
    struct prt_channel_s *channel;
    prt_direction_t direction;
    int agent;
} prt_callback_queue_t;

/**
 * @brief Callback data for releasing a device packet.
 */
typedef struct prt_callback_release_s {
    struct prt_vdp_s *vdp;
    struct prt_packet_s *packet;
} prt_callback_release_t;

prt_callback_finish_t *prt_callback_finish_new(
    struct prt_packet_s *src_packet,
    struct prt_packet_s *dst_packet,
    struct prt_channel_s *channel);
void prt_callback_finish_delete(prt_callback_finish_t *clbck);
void CUDART_CB prt_callback_finish_handler(
    cudaStream_t stream, cudaError_t status, void *dat);

prt_callback_queue_t *prt_callback_queue_new(
    struct prt_packet_s *old_packet,
    struct prt_packet_s *src_packet,
    struct prt_channel_s *channel,
    prt_direction_t direction,
    int agent);
void prt_callback_queue_delete(prt_callback_queue_t *clbck);
void CUDART_CB prt_callback_queue_handler(
    cudaStream_t stream, cudaError_t status, void *dat);

prt_callback_release_t *prt_callback_release_new(
    struct prt_vdp_s *vdp,
    struct prt_packet_s *packet);
void prt_callback_release_delete(prt_callback_release_t *callback);
void CUDART_CB prt_callback_release_handler(
    cudaStream_t stream, cudaError_t status, void *clbck);

#ifdef __cplusplus
}
#endif

#endif /* PRT_CALLBACK_H */
