/**
 * @file prt_config.c
 * @brief PRT configuration.
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.utk.edu/pulsar/
 * Copyright (C) 2012-2015 University of Tennessee.
 */

#include "prt_config.h"

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief Creates a new configuration object.
 *
 * @returns New configuration object with default values.
 */
prt_config_t* prt_config_new()
{
    // Allocate the config object.
    prt_config_t *config = (prt_config_t*)malloc(sizeof(prt_config_t));
    prt_assert(config != NULL, "malloc failed");

    // Set the defaults.
    config->vdp_scheduling = PRT_VDP_SCHEDULING_AGGRESSIVE;
    config->vdp_scheduling = PRT_SVG_TRACING_OFF;

    // Return the configuration object.
    return config;
}

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief Destroys a configuration object.
 *
 * @param config -- The configuration object to destroy.
 */
void prt_config_delete(prt_config_t *config)
{
    // Free the configuration object.
    free(config);
}
