/**
 * @file prt_device.h
 * @brief PRT device.
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.utk.edu/pulsar/
 * Copyright (C) 2012-2015 University of Tennessee.
 */

#ifndef PRT_DEVICE_H
#define PRT_DEVICE_H

#include "prt.h"

#ifdef __cplusplus
extern "C" {
#endif

struct prt_vsa_s;

/**
 * @brief VSA's accelerator device.
 * Represents a hardware accelerator.
 * Currently synonymous with an Nvidia GPU.
 *
 * "finished" is a one-directional synchronization variable.
 * Therefore declered volatile, but no need for atomic access.
 */
typedef struct prt_device_s {
    struct prt_vsa_s *vsa;  // Back link to the parent VSA.
    int rank;               // Local device number.
    int accelerator;        // Global accelerator number.
    icl_list_t *vdps;       // List of VDPs.
    icl_list_t *node;       // Current node in the VDP list.
    volatile int finished;  // Completion flag.
    int agent_rank;         // Rank of the communication agent.
} prt_device_t;

prt_device_t* prt_device_new(int rank, int accelerator, int agent_rank);
void prt_device_delete(prt_device_t *device);
void prt_device_cycle(prt_device_t *device);

#ifdef __cplusplus
}
#endif

#endif /* PRT_DEVICE_H */
