/**
 * @file prt_request.h
 * @brief PRT communication request.
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.utk.edu/pulsar/
 * Copyright (C) 2012-2015 University of Tennessee.
 */

#ifndef PRT_REQUEST_H
#define PRT_REQUEST_H

#include "prt.h"

#ifdef __cplusplus
extern "C" {
#endif

struct prt_packet_s;

/**
 * @brief MPI communication request for a packet.
 * Contains a packet, some info, MPI request and MPI status.
 */
typedef struct prt_request_s {
    struct prt_packet_s *packet;
    size_t size;
    int peer;
    int tag;
    MPI_Request request;
    MPI_Status status;
} prt_request_t;

prt_request_t *prt_request_new(
    struct prt_packet_s *packet, size_t size, int peer, int tag);
void prt_request_delete(prt_request_t *request);
void prt_request_send(prt_request_t *request);
void prt_request_recv(prt_request_t *request);
int prt_request_test(prt_request_t *request);
void prt_request_cancel(prt_request_t *request);

#ifdef __cplusplus
}
#endif

#endif /* PRT_REQUEST_H */
