/**
 * @file prt_thread.h
 * @brief PRT thread.
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.utk.edu/pulsar/
 * Copyright (C) 2012-2015 University of Tennessee.
 */

#ifndef PRT_THREAD_H
#define PRT_THREAD_H

#include "prt.h"

#ifdef __cplusplus
extern "C" {
#endif

struct prt_vsa_s;

/**
 * @brief VSA's worker thread.
 * Represents a single CPU core or a collection of cores.
 *
 * "finished" is a one-directional synchronization variable.
 * Therefore declered volatile, but no need for atomic access.
 */
typedef struct prt_thread_s {
    struct prt_vsa_s *vsa;  // Back link to the parent VSA.
    int rank;               // Local thread number.
    int core;               // Global core number.
    pthread_t id;           // Local pthread ID.
    icl_list_t *vdps;       // List of VDPs.
    volatile int finished;  // Completion flag.
    int agent_rank;         // Rank of the communication agent.
    double time;            // Life time.
} prt_thread_t;

prt_thread_t* prt_thread_new(int rank, int core, int agent_rank);
void prt_thread_delete(prt_thread_t *thread);
void* prt_thread_run(void *thrd);

#ifdef __cplusplus
}
#endif

#endif /* PRT_THREAD_H */
