/**
 * @file prt_transfer.c
 * @brief PRT local transfer.
 * @author Jakub Kurzak
 *
 * PULSAR Runtime
 * http://icl.utk.edu/pulsar/
 * Copyright (C) 2012-2015 University of Tennessee.
 */

#include "prt_transfer.h"

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief Creates a new local transfer object.
 *
 * @param packet -- The packet to transfer.
 * @param channel -- The channel to push to.
 * @param direction -- The direction of the transfer.
 * @param agent -- The number of the communication agent to use.
 *
 * @returns A new local transfer object.
 */
prt_transfer_t *prt_transfer_new(
    struct prt_packet_s *packet,
    struct prt_channel_s *channel,
    enum prt_direction_e direction,
    int agent)
{
    prt_transfer_t *transfer =
        (prt_transfer_t*)malloc(sizeof(prt_transfer_t));
    prt_assert(transfer != NULL, "malloc failed");
    transfer->packet = packet;
    transfer->channel = channel;
    transfer->direction = direction;
    transfer->agent = agent;
    return transfer;
}

////////////////////////////////////////////////////////////////////////////////
/**
 * @brief Destroys a local transfer object.
 *
 * @param -- The local transfer object to destroy.
 */
void prt_transfer_delete(prt_transfer_t *transfer)
{
    free(transfer);
}
